import weka.core.matrix.*;

/*
 * Octave:
 *
 * >> m = [.012350,-.054500,.015650;-.024850,.11920,.0048500;.019725,-.00065000,.35427]
 * m =
 *
 * 1.2350e-002  -5.4500e-002  1.5650e-002
 * -2.4850e-002  1.1920e-001  4.8500e-003
 * 1.9725e-002  -6.5000e-004  3.5427e-001
 *
 * >> sqrtm(m'*m)
 * ans =
 *
 * 6.7767e-003  -2.7023e-002  1.9569e-002
 * -2.7023e-002  1.2825e-001  4.9200e-005
 * 1.9569e-002  4.9200e-005  3.5411e-001
 *
 * >> pinv(sqrtm(m'*m))
 * ans =
 *
 * 3.6184e+006  7.6249e+005  -2.0007e+005
 * 7.6249e+005  1.6068e+005  -4.2159e+004
 * -2.0007e+005  -4.2159e+004  1.1065e+004
 *
 * >> pinv(m'*m)
 * ans =
 *
 * 1.3715e+013  2.8901e+012  -7.5832e+011
 * 2.8901e+012  6.0900e+011  -1.5980e+011
 * -7.5832e+011  -1.5980e+011  4.1928e+010
 */

public class Main
{
   public static void main(String[] args)
   {
      // Try Weka matrix.
      System.out.println("Weka matrix results:");
      //double[][] array = {{.00677670,-.02702334,-.02702334},{-.02702334,.1282538,.00004928},{.0195687,.00004931,.35411346}};
      double[][] array = { { .012350, -.054500, .015650 }, { -.024850, .11920, .0048500 }, { .019725, -.00065000, .35427 } };
      //m = [.012350,-.054500,.015650;-.024850,.11920,.0048500;.019725,-.00065000,.35427]
      weka.core.matrix.Matrix A = new weka.core.matrix.Matrix(array);
      System.out.println("source:");
      A.print(8, 5);
      weka.core.matrix.Matrix a   = A.transpose();
      weka.core.matrix.Matrix AtA = a.times(A);
      System.out.println("AtA:");
      AtA.print(8, 5);
      weka.core.matrix.Matrix Sqrt = AtA.sqrt();
      System.out.println("sqrtm:");
      Sqrt.print(8, 5);
      weka.core.matrix.Matrix Inv = AtA.inverse();
      System.out.println("pinv:");
      Inv.print(8, 5);
      System.out.println("pinv(sqrtm):");
      weka.core.matrix.Matrix C = Sqrt.inverse();
      C.print(8, 5);
      System.out.println("det=" + Sqrt.det());
      System.out.println();

      // Try local Weka matrix.
      System.out.println("Local Weka matrix results:");
      //double[][] array = {{.00677670,-.02702334,-.02702334},{-.02702334,.1282538,.00004928},{.0195687,.00004931,.35411346}};
      //m = [.012350,-.054500,.015650;-.024850,.11920,.0048500;.019725,-.00065000,.35427]
      weka.Matrix A2 = new weka.Matrix(array);
      System.out.println("source:");
      A2.print(8, 5);
      weka.Matrix a2   = A2.transpose();
      weka.Matrix AtA2 = a2.times(A2);
      System.out.println("AtA2:");
      AtA2.print(8, 5);
      weka.Matrix Sqrt2 = AtA2.sqrt();
      System.out.println("sqrtm:");
      Sqrt2.print(8, 5);
      weka.Matrix Inv2 = AtA2.inverse();
      System.out.println("pinv:");
      Inv2.print(8, 5);
      System.out.println("pinv(sqrtm):");
      weka.Matrix C2 = Sqrt2.inverse();
      C2.print(8, 5);
      System.out.println("det=" + Sqrt2.det());
      System.out.println();

      // Try our matrix.
      System.out.println("Our matrix results:");
      Matrix m = Matrix.matrix_alloc(3, 3);
      //m = [.012350,-.054500,.015650;-.024850,.11920,.0048500;.019725,-.00065000,.35427]
      m.e[0] = .012350;
      m.e[1] = -.054500;
      m.e[2] = .015650;
      m.e[3] = -.024850;
      m.e[4] = .11920;
      m.e[5] = .0048500;
      m.e[6] = .019725;
      m.e[7] = -.00065;
      m.e[8] = .35427;

      //double[][] array = {{.00677670,-.02702334,-.02702334},{-.02702334,.1282538,.00004928},{.0195687,.00004931,.35411346}};

      /*
       * m.e[0] = .00677670;
       * m.e[1] = -.02702334;
       * m.e[2] = -.02702334;
       * m.e[3] = -.02702334;
       * m.e[4] = .1282538;
       * m.e[5] = .00004928;
       * m.e[6] = .0195687;
       * m.e[7] = .00004931;
       * m.e[8] = .35411346;
       */
      System.out.println("source:");
      Matrix.matrix_print(m);
      System.out.println();
      Matrix mt = Matrix.matrix_alloc(3, 3);
      Matrix.matrix_transpose(m, mt);
      Matrix mtm = Matrix.matrix_alloc(3, 3);
      Matrix.matrix_multiply(mt, m, mtm);
      System.out.println("mtm:");
      Matrix.matrix_print(mtm);
      System.out.println();
      Matrix msqrt = Matrix.matrix_alloc(3, 3);
      Matrix.matrix3_sqrt(mtm, msqrt);
      System.out.println("sqrtm:");
      Matrix.matrix_print(msqrt);
      System.out.println();
      Matrix mpinv = Matrix.matrix_alloc(3, 3);
      Matrix.matrix3_pinv(mtm, mpinv);
      System.out.println("pinv:");
      Matrix.matrix_print(mpinv);
      System.out.println();
      Matrix mpinvsqrt = Matrix.matrix_alloc(3, 3);
      Matrix.matrix3_pinv(msqrt, mpinvsqrt);
      System.out.println("pinv(sqrt):");
      Matrix.matrix_print(mpinvsqrt);
      System.out.println();
      Matrix minvsqrt = Matrix.matrix_alloc(3, 3);
      Matrix.matrix3_inv(msqrt, minvsqrt);
      System.out.println("inv(sqrt):");
      Matrix.matrix_print(minvsqrt);
   }
}
