// vector3d structure
public class Vector3d
{
   public double x, y, z;

   //allocate matrix (m x n)
   Vector3d(double x, double y, double z)
   {
      this.x = x;
      this.y = y;
      this.z = z;
   }


   //allocate vector
   public static Vector3d vector3d_alloc()
   {
      return(new Vector3d(0.0, 0.0, 0.0));
   }


   //get length of a 3d vector sqrt(x^2+y^2+y^2)
   public static double vector3d_length(Vector3d vector)
   {
      double R;

      R  = vector.x * vector.x;
      R += vector.y * vector.y;
      R += vector.z * vector.z;
      return(Math.sqrt(R));
   }


   //convert vector to a vector with same direction and modulus 1
   public static void vector3d_normalize(Vector3d vector)
   {
      double R;

      R         = vector3d_length(vector);
      vector.x /= R;
      vector.y /= R;
      vector.z /= R;
   }


   //calculate vector dot-product  c = a . b
   public static double vector3d_dot(Vector3d a, Vector3d b)
   {
      return(a.x * b.x + a.y * b.y + a.z * b.z);
   }


   //calculate vector cross-product  c = a x b
   public static void vector3d_cross(Vector3d a, Vector3d b, Vector3d c)
   {
      c.x = a.y * b.z - a.z * b.y;
      c.y = a.z * b.x - a.x * b.z;
      c.z = a.x * b.y - a.y * b.x;
   }


   //calculate vector scalar-product  b = s x a
   public static void vector3d_scale(double s, Vector3d a, Vector3d b)
   {
      b.x = s * a.x;
      b.y = s * a.y;
      b.z = s * a.z;
   }


   //calculate vector sum   c = a + b
   public static void vector3d_add(Vector3d a, Vector3d b, Vector3d c)
   {
      c.x = a.x + b.x;
      c.y = a.y + b.y;
      c.z = a.z + b.z;
   }


   //copy a to b
   public static void vector3d_copy(Vector3d a, Vector3d b)
   {
      b.x = a.x;
      b.y = a.y;
      b.z = a.z;
   }
}
